/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_proftpd.c -- proftpd-related colorizers for CCZE
 * Copyright (C) 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <stdlib.h>
#include <string.h>

static void ccze_proftpd_setup (void);
static void ccze_proftpd_shutdown (void);
static int ccze_proftpd_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_proftpd_access, *reg_proftpd_auth;

static char *
ccze_proftpd_access_log_process (pcre2_match_data *offsets)
{
  char *host, *user, *auser, *date, *command, *file, *ftpcode, *size;
  size_t l;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&host, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&user, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&auser, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&command, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&file, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&ftpcode, &l);
  pcre2_substring_get_bynumber (offsets, 8, (unsigned char **)&size, &l);

  ccze_addstr (CCZE_COLOR_HOST, host);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_USER, user);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_USER, auser);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_DEFAULT, "[");
  ccze_print_date (date);
  ccze_addstr (CCZE_COLOR_DEFAULT, "]");
  ccze_space ();

  ccze_addstr (CCZE_COLOR_DEFAULT, "\"");
  ccze_addstr (CCZE_COLOR_KEYWORD, command);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_URI, file);
  ccze_addstr (CCZE_COLOR_DEFAULT, "\"");
  ccze_space ();

  ccze_addstr (CCZE_COLOR_FTPCODES, ftpcode);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_GETSIZE, size);
  
  ccze_newline ();

  pcre2_substring_free (size);
  pcre2_substring_free (ftpcode);
  pcre2_substring_free (file);
  pcre2_substring_free (command);
  pcre2_substring_free (date);
  pcre2_substring_free (auser);
  pcre2_substring_free (user);
  pcre2_substring_free (host);

  return NULL;
}

static char *
ccze_proftpd_auth_log_process (pcre2_match_data *offsets)
{
  char *servhost, *pid, *remhost, *date, *cmd, *value, *ftpcode;
  size_t l;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&servhost, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&pid, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&remhost, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&cmd, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&value, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&ftpcode, &l);

  ccze_addstr (CCZE_COLOR_HOST, servhost);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, "ftp server");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_PIDB, "[");
  ccze_addstr (CCZE_COLOR_PID, pid);
  ccze_addstr (CCZE_COLOR_PIDB, "]");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_HOST, remhost);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, "[");
  ccze_print_date (date);
  ccze_addstr (CCZE_COLOR_DEFAULT, "]");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, "\"");
  ccze_addstr (CCZE_COLOR_KEYWORD, cmd);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, value);
  ccze_addstr (CCZE_COLOR_DEFAULT, "\"");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, ftpcode);
  
  ccze_newline ();
    
  pcre2_substring_free (ftpcode);
  pcre2_substring_free (value);
  pcre2_substring_free (cmd);
  pcre2_substring_free (date);
  pcre2_substring_free (remhost);
  pcre2_substring_free (pid);
  pcre2_substring_free (servhost);

  return NULL;
}

static void
ccze_proftpd_setup (void)
{
  int error;
  size_t errptr;

  reg_proftpd_access = pcre2_compile
    ("^(\\d+\\.\\d+\\.\\d+\\.\\d+) (\\S+) (\\S+) "
     "\\[(\\d{2}/.{3}/\\d{4}:\\d{2}:\\d{2}:\\d{2} [\\-\\+]\\d{4})\\] "
     "\"([A-Z]+) ([^\"]+)\" (\\d{3}) (-|\\d+)", PCRE2_ZERO_TERMINATED,
     0, &error, &errptr, NULL);

  reg_proftpd_auth = pcre2_compile
    ("^(\\S+) ftp server \\[(\\d+)\\] (\\d+\\.\\d+\\.\\d+\\.\\d+) "
     "\\[(\\d{2}/.{3}/\\d{4}:\\d{2}:\\d{2}:\\d{2} [\\-\\+]\\d{4})\\] "
     "\"([A-Z]+) ([^\"]+)\" (\\d{3})", PCRE2_ZERO_TERMINATED,
     0, &error, &errptr, NULL);
}

static void
ccze_proftpd_shutdown (void)
{
  pcre2_code_free (reg_proftpd_auth);
  pcre2_code_free (reg_proftpd_access);
}

static int
ccze_proftpd_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;

  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_proftpd_access, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_proftpd_access_log_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }
  if (pcre2_match (reg_proftpd_auth, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_proftpd_auth_log_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);

  return 0;
}

CCZE_DEFINE_PLUGIN (proftpd, FULL,
		    "Coloriser for proftpd access and auth logs.");
